#!/usr/bin/env python
"""
    nmapy module
    version: 0.5a
    author: Dan Hickox
    date: 03/16/13
"""

import urllib2
import xml.dom.minidom
import syslog


def __main():
    """
        called when ran standalone
    """
    global cmdline_args
    
    cmdline_args = __get_cmdline_args()
    if not cmdline_args:
        print( "main: Fail - variable cmdline_args is None " )
        sys.exit( EXIT_FAILURE )
    
    
    nma = Nma()
    result = None
    if cmdline_args.mode == __NOTIFY: 
        result = nma.notify( cmdline_args.apikey,
                             cmdline_args.application,
                             cmdline_args.event,
                             cmdline_args.description,
                             cmdline_args.priority,
                             cmdline_args.developerkey,
                             cmdline_args.url,
                             cmdline_args.content_html )
    elif cmdline_args.mode == __VERIFY:
        result = nma.verify( cmdline_args.apikey[ 0 ],
                             cmdline_args.developerkey )
    
    if result == True:
        if not cmdline_args.quiet:
            if cmdline_args.mode == __NOTIFY:
                print( "main: notification sent" )
            elif cmdline_args.mode == __VERIFY:
                print( "main: apikey valid/active" )
    elif result == False:
        if not cmdline_args.quiet:
            if cmdline_args.mode == __NOTIFY:
                print( "main: notification failed" )
            elif cmdline_args.mode == __VERIFY:
                print( "main: apikey invalid/inactive" )
    
    if cmdline_args.debug:
        print( "main: arguments: %s" % str( cmdline_args ) )
        print( "main: nma errors %s" % str( nma.errors ) )
        print( "main: nma status codes %s" % str( nma.response_code ) )
        print( "main: nma request_text %s" % str( nma.request_text ) )
    
    if result is None:
        if not cmdline_args.quiet:
            print( "main: transaction state unknown. failed." )
        sys.exit( EXIT_FAILURE )
    
    sys.exit( EXIT_SUCCESS )

def __get_cmdline_args():
    """
        return argparse namespace object
    """
    parser_obj = argparse.ArgumentParser()
    
    parser_obj.add_argument( "--mode",
                            choices     = ( "verify", "notify" ),
                            required    = True,
                            action      = "store",
                            help        = "set NMA API mode" )
    
    parser_obj.add_argument( "--apikey",
                            required    = True,
                            action      = "append",
                            help        = "NMA API keys" )
                            
    parser_obj.add_argument( "--application",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Application that is notifying ( 256 char max )" )
                            
    parser_obj.add_argument( "--event",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Event type description ( 1000 char max )" )
                            
    parser_obj.add_argument( "--description",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Notification text ( 10000 char max )" )
                            
    parser_obj.add_argument( "--priority",
                            required    = False,
                            action      = "store",
                            type        = int,
                            default     = 0,
                            help        = "Priority low to high ( -2 -1 0 1 2 )" )
    
    parser_obj.add_argument( "--developerkey",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "Developer API Key" )
                            
    parser_obj.add_argument( "--url",
                            required    = False,
                            action      = "store",
                            default     = None,
                            help        = "An URL/URI can be attached to your notification. ( 2000 char max )" )
                            
    parser_obj.add_argument( "--content-html",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Flags NMA to handle a subset of html tags" )
                            
    parser_obj.add_argument( "-d", "--debug",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Enable debug output" )
                            
    parser_obj.add_argument( "-q", "--quiet",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Suppress all output" )
                            
    parser_obj.add_argument( "-s", "--syslog",
                            required    = False,
                            action      = "store_true",
                            default     = False,
                            help        = "Send output to syslog instead of stdout" )
                            
    return parser_obj.parse_args()


class Nma( object ):
    """
        nma class description
    """   
    __RESPONSE_CODES = {
                    "verify" : {
                        "200" : "API key is valid",
                        "400" : "The data supplied is in the wrong format, invalid length or null",
                        "401" : "None of the API keys are valid",
                        "402" : "Maximum number of API calls per hour exceeded",
                        "500" : "Internal server error" },
                    "notify" : {
                        "200" : "Notification submitted",
                        "400" : "The data supplied is in the wrong format, invalid length or null",
                        "401" : "None of the API keys are valid",
                        "402" : "Maximum number of API calls per hour exceeded",
                        "500" : "Internal server error" }
                    }
        
    __VERIFY = "verify"
    __NOTIFY = "notify"
    
    __MODES = ( __VERIFY, __NOTIFY )
    
    __APIKEY_LEN = 48
    __APPLICATION_TEXT_MAX_LEN = 256
    __EVENT_TEXT_MAX_LEN = 1000
    __DESCRIPTION_TEXT_MAX_LEN = 10000
    __URL_TEXT_MAX_LEN = 2000
    
    __PRIORITY_MAP = {
                    -2 : "Very Low",
                    -1 : "Moderate",
                     0 : "Normal",
                     1 : "High",
                     2 : "Emergency" }
    
    __API_HOST = "notifymyandroid.com"
    
    __API_PATH_MAP = {
                    __VERIFY : "/publicapi/verify",
                    __NOTIFY : "/publicapi/notify" }
    
    def __init__( self ):
        """request/response"""
        self.request_data   = None
        self.request_text   = ""
        self.fresponse_data = None
        self.response_code  = ""
        self.response_text  = ""
        """logging"""
        self.errors         = []
        self.syslog         = False
 
    def notify( self, apikeys, application,
                event, description, priority = 0,
                developerkey = None, url = None, content_html = False ):
        """
            notify
            @apikeys: A list of NMA API keys
            @application: string description of app sending the notification
            @event: string description of event type.
            @description: long description or message
            @priority: priority defaults to ( 0 == normal )
            @developerkey: optional developerkey
            @url: optional url to send within the message
            @content_html: option to enable a subset of html tags
            sets self.response_code and self.response_text
            returns bool
        """
        """api vars"""
        self.mode           = self.__NOTIFY 
        self.apikey         = apikeys
        self.application    = application
        self.event          = event
        self.description    = description
        self.priority       = priority
        self.developerkey   = developerkey
        self.url            = url
        self.content_html   = content_html

        if not isinstance( self.apikey, list ):
            self.errors.append( ( "notify: apikeys not list", type( self.apikey ) ) )
            return False
        if not len( self.apikey ):
            self.errors.append( ( "notify: apikeys empty list", len( self.apikey ) ) )
            return False

        for key in self.apikey:
            if not self.__is_key_valid( key ):
                self.errors.append( ( "notify: key %s not valid format" % key, len( key ) ) )
                return False

        if not self.__is_msg_valid():
            return False
        if not self.__set_request_text():
            return False
        if not self.__send_request():
            return False
        if not self.__parse_xml_response():
            self.errors.append( ( "notify: unable to parse response xml", None ) )
                        
        return True
 
    def verify( self, apikey, developerkey = None ):
        """
            implements NMA API verify
            @apikey: a single NMA API key as string
            @developerkey: optional developerkey
            sets self.response_code and self.response_text
            returns bool
        """
        if not self.__is_key_valid( apikey ):
            self.errors.append( ( "verify: apikey %s invalid format" % str( apikey ), None ) )
            return False
            
        self.apikey = [ apikey ]
        
        if developerkey and not self.is_key_valid( developerkey ):
            self.errors.append( ( "verify: developer key %s invalid format" % str( developerkey ), None ) )
            return False
        
        self.developerkey = developerkey
        
        self.mode = self.__VERIFY
        if not self.__set_request_text():
            return False
        if not self.__send_request():
            return False
        if not self.__parse_xml_response():
            self.errors.append( ( "verify: unable to parse response xml", None ) )
        
        return True
        
        
    def enable_syslog( self ):
        """
            enable logging to syslog
            return bool
        """
        pass
        
    def __is_key_valid( self, apikey ):
        """
        verify the key's format is valid
        return bool
        """
        if not isinstance( apikey, basestring ):
            self.errors.append( ( "__check_apikey: unknown type", type( apikey ) ) )
            return False
        if len( apikey ) != self.__APIKEY_LEN:
            self.errors.append( ( "__check_apikey: key %s incorrect length" % apikey, len( apikey ) ) )
            return False
        return True

    def __is_msg_valid( self ):
        """
        check values are as expected.
        return bool
        """
        if isinstance( self.application, basestring ):
            if len( self.application ) > self.__APPLICATION_TEXT_MAX_LEN:
                self.errors.append( ( "application text max len surpassed", len( self.application ) ) )
        else:
            self.errors.append( ( "application not string", type( self.application ) ) )
        if isinstance( self.event, basestring ):
            if len( self.event ) > self.__EVENT_TEXT_MAX_LEN:
                self.errors.append( ( "event text max len surpassed", len( self.event ) ) )
        else:
            self.errors.append( ( "event not string", type( self.event ) ) )
        if isinstance( self.description, basestring ):
            if len( self.description ) > self.__DESCRIPTION_TEXT_MAX_LEN:
                self.errors.append( ( "description text max len surpassed", len( self.description ) ) )
        else:
            self.errors.append( ( "description not string", type( self.description ) ) )
        if isinstance( self.priority, int ):
            if self.priority not in self.__PRIORITY_MAP.keys():
                self.errors.append( ( "unknown priority", self.priority ) )
        else:
            self.errors.append( ( "priority unknown type", type( self.priority ) ) )
        if self.url:
            if isinstance( self.url, basestring ):
                if len( self.url ) > self.__URL_TEXT_MAX_LEN:
                    self.errors.append( ( "url max len surpassed", len( self.url ) ) )
            else:
                self.errors.append( ( "url type unknown", type( self.url ) ) )
        
        if len( self.errors ):
            return False
        
        return True

        
    def __set_request_text( self ):
        """
        ensure data being sent is utf-8
        return bool
        """
        if self.mode == self.__VERIFY:
            self.request_data = [ ( "apikey", ",".join( self.apikey ) ) ]
            if self.developerkey:
                self.request_data.extend( [ ( "developerkey", self.developerkey ) ] )
        elif self.mode == self.__NOTIFY:
            self.request_data = [ ( "apikey", ",".join( self.apikey ) ),
                                  ( "application", self.application ),
                                  ( "event", self.event ),
                                  ( "description", self.description ) ]
            if self.priority:
                self.request_data.extend( [ ( "priority", str( self.priority ) ) ] )
            if self.developerkey:
                self.request_data.extend( [ ( "developerkey", self.developerkey ) ] )
            if self.url:
                self.request_data.extend( [ ( "url", self.url ) ] )
            if self.content_html:
                self.request_data.extend( [ ( "content-type", "text/html" ) ] )
        
        param_list = []
        for t in self.request_data:
            param_list.append( "=".join( t ) )
        
        self.request_text = "&".join( param_list ).encode( "utf_8" )
        
        return True

    
    def __send_request( self ):
        """
        try notification
        return bool
        """ 
        self.fresponse_data = urllib2.urlopen( ( "https://" + self.__API_HOST + self.__API_PATH_MAP[ self.mode ] ),
                                            urllib2.quote( self.request_text, safe="&=" ) )
        if not self.fresponse_data:
            return False
        return True

        
    def __parse_xml_response( self ):
        """
        attempt parsing the response and
        set response_code/response_text
        return bool
        """
        self.response_code = ""
        self.response_text = ""
        xmldom = xml.dom.minidom.parse( self.fresponse_data )
        if not xmldom:
            self.errors.append( "xmldom is None" )
            return False
        if not xmldom.hasChildNodes():
            self.errors.append( "xmldom has no children" )
            return False
        nma = None
        for node in xmldom.childNodes:
            if node.nodeType == node.TEXT_NODE:
                continue
            if node.nodeName == "nma":
                nma = node
                break
        if nma is None:
            self.errors.append( "unable to set nma node" )
            return False
        if not nma.hasChildNodes():
            self.errors.append( "nma has no children" )
        status = None
        for node in nma.childNodes:
            if node.nodeType == node.TEXT_NODE:
                continue
            if node.nodeName == "success" or node.nodeName == "error":
                status = node
                break
        if not status:
            self.errors.append( "unable to set status node" )
            return False
        if not status.hasAttributes():
            self.errors.append( "status node has no attributes" )
            return False
        for k,v in status.attributes.items():
            if k == "code":
                self.response_code = v
                self.response_text = self.__RESPONSE_CODES[ self.mode ][ v ]
                return True
        
        return False
        
        

if __name__ == "__main__":

    import sys
    import argparse

    __VERIFY = "verify"
    __NOTIFY = "notify"

    EXIT_SUCCESS = 0
    EXIT_FAILURE = 1

    cmdline_args = None
    
    try:
        __main()
    except KeyboardInterrupt:
        sys.exit( EXIT_SUCCESS )
    except Exception as e:
        if not cmdline_args.quiet:
            print( "main: failed to run with error: %s" % str( e ) )
        sys.exit( EXIT_FAILURE )
    
